
/* cwds header file 

Copyright (C) 2009 Roberto Navone <ron@object65.it>
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; version 2
of the License.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
*/

#ifdef __cplusplus
extern "C" {
#endif

/*
 * ---------------
 * basic functions 
 * --------------- 
 */

/*
 * Library initialization
 *
 * smpData pointer to binary translated samples definition file, 0 select the built in definition file
 * smpDataLen len of binary translated samples definition file, not used if built in definition file is selected
 */
void cwdsInit(unsigned char *smpData, unsigned long smpDataLen);

/*
 * Enter in recognize mode
 *
 * w width of cell area
 * h height of cell area
 */
void cwdsBeginRecognizeMode(int w, int h);

/*
 * Notify a pen down event in cella area
 *
 * x, y event coordinates
 */
void cwdsNotifyPenDownEvent(int x, int y);

/*
 * Notify a pen move event in cell area 
 *
 * x, y event coordinates
 */
void cwdsNotifyPenMoveEvent(int x, int y);

/*
 * Notify a pen up event in cell area
 */
void cwdsNotifyPenUpEvent();

/*
 * Terminate recognize mode and perform recognition
 *
 * returns the recognized symbol, -1 in no recognition done
 */
int cwdsEndRecognizeMode();

/*
 * Abandon the recognize mode 
 */
void cwdsCancelRecognizeMode();

/* 
 * --------------------------------
 * extended functions - recognition 
 * --------------------------------
 */

/*
 * Enable the requested symbol
 */
void cwdsEnableSymbol(int ch);

/*
 * Enable symbols in requeste range
 */
void cwdsEnableSymbolRange(int chFrom, int chTo);

/*
 * Disable the requested symbol
 */
void cwdsDisableSymbol(int ch);

/*
 * Disable symbols in requested range
 */
void cwdsDisableSymbolRange(int chFrom, int chTo);

/*
 * Returns the max number of recognition alternatives
 */
int cwdsGetMaxRecognitionAlts();

/*
 * Return the time spent for the last recognition, in msec
 */
int cwdsGetLastRecognitionTime();

/*
 * Get the number of possible alternatives in the last recognition phase
 */
int cwdsGetLastRecognitionNumAlts();

/*
 * Get informations about the requested alternative for the last recognition phase
 *
 * nAlt the alternative 
 *
 * returns ch the alternative symbol
 * returns nSample the sample used for the recognition
 * returns pct the percentage for the recognition
 */
void cwdsGetLastRecognitionAltInfo(int nAlt, int *ch, int *nSample, int *pct);

/* 
 * ---------------------------------------------
 * extended functions - training & samples store 
 * ---------------------------------------------
 */

/*
 * Enter in training mode
 *
 * ch the symbol to train
 * w width of cell area
 * h height of cell area
 */
void cwdsBeginTrainingMode(int ch, int w, int h);

/*
 * Terminate training mode and store symbol definition
 */
void cwdsEndTrainingMode();

/*
 * Abandon the training mode 
 */
void cwdsCancelTrainingMode();

/* 
 * Save samples
 *
 * fileName samples file name in fat lib style as "fat:/hb/cwsamples.dat"
 */
void cwdsSaveSamples(char *fileName);

/*
 * Returns the number of samples defined for the requested symbol
 */
int cwdsGetNumSamples(int ch);

/*
 * Reset all the samples defined for the requested symbol
 */
void cwdsResetSamples(int ch);

/*
 * Get the number of graphics commands for the requested sample of the requested symbol
 */
int cwdsGetSampleNumGraphicsCommands(int ch, int nSample);

/*
 * Returns the requeste graphic command for the requested sample of the requested symbol
 *
 * w the cell's drawing area width
 * h the cell's drawing area height
 *
 * returns graphic command coordinates within the cell's area:
 * if x2, y2 are -1 the command is a point (x1, y1) else the command is a line (x1, y1 to x2, y2)
 */
void cwdsGetSampleGraphicCommand(int ch, int nSample, int nCommand, int w, int h, int *x1, int *y1, int *x2, int *y2);

#ifdef __cplusplus
}
#endif
