
/* CellWrite implementation 

Copyright (C) 2009 Roberto Navone <ron@object65.it>
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; version 2
of the License.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
*/

#include <nds.h>
#include <stdio.h>
#include <string.h>

#define CWDATA
#include "CellWriter.h"
#include "CellWriterForm.h"
#include "CellWriterTrainForm.h"
#include "CellWriterWriteForm.h"
#include "LiteralHandler.h"

#include "cwds_bmp_dat.h"
#include "flagita_bmp_dat.h"
#include "flageng_bmp_dat.h"
#include "info_bmp_dat.h"

static Bitmap cwdsBmp, flagitaBmp, flagengBmp, infoBmp;

/*
 * Constructor
 */
CellWriter::CellWriter()
{
  GraphicsHandler::MakeBitmap(&cwdsBmp, cwds_bmp_dat, false, 0);
  GraphicsHandler::MakeBitmap(&flagitaBmp, flagita_bmp_dat, true, 0x0F95);
  GraphicsHandler::MakeBitmap(&flagengBmp, flageng_bmp_dat, true, 0x0F95);
  GraphicsHandler::MakeBitmap(&infoBmp, info_bmp_dat, true, 0x0F95);
}

/*
 * Run application
 */
void CellWriter::Run(TimersHandler *th)
{
  /* Create graphics handler */
  
  MainGraphicsHandler* mgh = new MainGraphicsHandler();
  
  SubGraphicsHandler* sgh = new SubGraphicsHandler();
  
  CellWriterEngine* eng = new CellWriterEngine(mgh, sgh);
  
  /* Create CellWriter engine */
  
  /* Display intro and select working language */
  
  int lang = SelectWorkingLanguage(mgh, sgh);
  
  /* Set language */
  
  LiteralHandler::SetLanguage(lang);
  
  /* Run cell writer */
  
  RunCellWriter(th, mgh, sgh, eng);
  
  /* Free handlers */

  delete eng;
  delete sgh;
  delete mgh;
}

/*
 * Select working language
 */
int CellWriter::SelectWorkingLanguage(GraphicsHandler* mgh, GraphicsHandler* sgh)
{
  mgh->ClearScreen();
  
  sgh->DrawBitmap(0, 0, &cwdsBmp);
  
  //sgh->ClearScreen(BCK_COLOR);
  
  /* Draw logo un upper screen */
  
  mgh->DrawTextTrueType(GraphicsHandler::TTF_dinobl, LIT_TITLE, -1, Y_TITLE, H_TITLE, 0xffff, 0x0000);
  mgh->DrawTextTrueType(GraphicsHandler::TTF_dinobl, "handwriting library", -1, Y_SUBTITLE1, H_SUBTITLE, 0xffff, 0x0000);
  mgh->DrawTextTrueType(GraphicsHandler::TTF_dinobl, "test program", -1, Y_SUBTITLE2, H_SUBTITLE, 0xffff, 0x0000);
  mgh->DrawTextTrueType(GraphicsHandler::TTF_goulong, LIT_WEBSITE, -1, 190, 11, 0xffff, 0x0000);
  
  /* Draw language & info buttons on lower screen... */
  
  ButtonsHandler* bh = new ButtonsHandler(sgh);
  
  bh->AddButtonArea(IDB_ITALIAN, XBTN_ITA, YBTN_LANG, WBTN_LANG, HBTN_LANG, &flagitaBmp);
  bh->AddButtonArea(IDB_ENGLISH, XBTN_ENG, YBTN_LANG, WBTN_LANG, HBTN_LANG, &flagengBmp);

  bh->AddButtonArea(IDB_INFO, XBTN_INFO, YBTN_INFO, WBTN_INFO, HBTN_INFO, &infoBmp);
  
  /* ... and wait for language selection */
  
  int idb;
  
  bool bPenDown = false;
  
  touchPosition touch;
  
  bool bInfoShow = false;
  
  while (true)
  {
    scanKeys();
    
    if(keysHeld() & KEY_TOUCH)
		{
      touch = touchReadXY();
                   
      if (touch.px != 0  &&  touch.py != 0)
      {
        if (!bPenDown)
        {
          bPenDown = true;
          
          Button* btn = bh->CheckButtonAt(touch.px, touch.py);
          
          if (btn != 0)
          {
            idb = btn->GetId();
            
            if (idb == IDB_INFO)
            {
              if (!bInfoShow)
              {
                  mgh->DrawTextTrueType(GraphicsHandler::TTF_goulong, "Version 1.0, Copyright(C) 2009 Roberto Navone.", 5, 123, 11, 0xffff, 0x0000);
                  mgh->DrawTextTrueType(GraphicsHandler::TTF_goulong, "CellWriterDS comes with ABSOLUTELY NO", 5, 137, 11, 0xffff, 0x0000);
                  mgh->DrawTextTrueType(GraphicsHandler::TTF_goulong, "WARRANTY. This program is free software;", 5, 149, 11, 0xffff, 0x0000);
                  mgh->DrawTextTrueType(GraphicsHandler::TTF_goulong, "licensed under the terms of the GNU General", 5, 161, 11, 0xffff, 0x0000);
                  mgh->DrawTextTrueType(GraphicsHandler::TTF_goulong, "Public License; version 2.", 5, 173, 11, 0xffff, 0x0000);

                bInfoShow = true;
              }
                
              continue;
            }
            
            break;
          }
          
        } 
      }
    }
    else
      bPenDown = false;
      
    swiWaitForVBlank();
  }

  /* Ok, proceed */
  
  delete bh;
  
  return (idb == IDB_ITALIAN ? ITALIAN : ENGLISH);
}

/*
 * Run CellWRiter
 */
void CellWriter::RunCellWriter(TimersHandler *th, MainGraphicsHandler* mgh, SubGraphicsHandler* sgh, CellWriterEngine* eng)
{
  RunMode mode = WRITE;
  
  CellWriterForm::RunResult rc;
  
  
  while (true)
  {
    CellWriterForm* cwf = 0;
    
    switch (mode)
    {
      case TRAIN:
        cwf = new CellWriterTrainForm(eng, mgh, sgh, th);
        break;
        
      case WRITE:
        cwf = new CellWriterWriteForm(eng, mgh, sgh, th);
        break;
    }
    
    rc = cwf->Run();
    
    delete cwf;
    
    if (rc == CellWriterForm::EXIT)
      break;
      
    mode = (mode == TRAIN) ? WRITE : TRAIN;
  }
}
