
/* CellWriterEngine declaration 

Copyright (C) 2009 Roberto Navone <ron@object65.it>
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; version 2
of the License.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
*/

#ifndef CELLWRITERENGINEH
#define CELLWRITERENGINEH

#include "GraphicsHandler.h"

class CellWriterEngine
{
public:

  /*
   * Constructor
   */
  CellWriterEngine(MainGraphicsHandler* mgh, SubGraphicsHandler* sgh);
  
  /*
   * Destructor
   */
  ~CellWriterEngine();
  
  /*
   * Check if in training mode
   */
  bool IsInTrainingMode();
  
  /*
   * Check if in recognize mode
   */
  bool IsInRecognizeMode();
  
  /*
   * Begin training mode for the requested char
   */
  void BeginTrainingMode(int ch, int cellX, int cellY, int cellW, int cellH, unsigned short color);
  
  /*
   * Begin recognize mode
   */
  void BeginRecognizeMode(int cellX, int cellY, int cellW, int cellH, unsigned short color);
  
  /*
   * Button depressed in cell area
   */
  void NotifyButtonPressEvent(int x, int y);
  
  /*
   * Motion with button depressed in cell area
   */
  void NotifyMotionEvent(int x, int y);
  
  /*
   * Button released
   */
  void NotifyButtonReleaseEvent();
  
  /*
   * End current training mode
   */
  void EndTrainingMode();
  
  /*
   * Cancel current training mode
   */
  void CancelTrainingMode();
  
  /*
   * End current recognize mode
   */
  int EndRecognizeMode();
  
  /*
   * Cancel current recognize mode
   */
  void CancelRecognizeMode();
  
  /*
   * Enable/disable the requested sample
   */
  void EnableSample(int ch, bool bEnable);

  /*
   * Enable samples in requeste range
   */
  void EnableSampleRange(int chFrom, int chTo, bool bEnable);

  /*
   * Get the number of samples for the requested char
   */
  int GetNumSamples(int ch);
  
  /*
   * Reset the samples for the requested char
   */
  void ResetSamples(int ch);
  
  /*
   * Draw the requested sample for the requested char
   */
  void DrawSample(int ch, int nSample, bool mainDisplay, int cellX, int cellY, int cellW, int cellH, unsigned short color);
  
  /*
   * Return the recognition time for the last recognized char
   */
  int GetRecognitionTime();
  
  /*
   * Get the max number of recognition alternatives
   */
  int GetMaxRecognitionAlts();
  
  /*
   * Get the number of recognition alternatives
   */
  int GetNumRecognitionAlts();
  
  /*
   * Get the nth recognition alternative informations
   */
  void GetRecognitionAltInfo(int nAlt, int *ch, int *nSample, int *pct);
  
  /*
   * Save samples
   */
  void SaveSamples(char *fileName);
  
private:

  /*
   * Draw the requested sample
   */
  void DrawSample(void *sample, int cellX, int cellY, int cellW, int cellH, unsigned short color);
  
  /*
   * Render requested point
   */
  void render_point(bool mainDisplay, int x, int y, unsigned short color);
  
  /*
   * Render requested segment
   */
  void render_segment(bool mainDisplay, int x1, int y1, int x2, int y2, unsigned short color);
  
  /* 
   * Engine modes
   */
  enum Mode
  {
    None,
    Training,
    Recognize,
  };
  
  /*
   * Current engine mode
   */
  Mode m_mode;
  
  /*
   * Working char
   */
  int m_ch;
  
  /*
   * Working cell coords
   */
  int m_cellX, m_cellY, m_cellW, m_cellH;
  
  /*
   * Last coordinates
   */
  int m_lastX, m_lastY;
  
  /*
   * Draw color
   */
  unsigned short m_color;
  
  /*
   * Graphics handlers
   */
  MainGraphicsHandler* m_mgh;
  SubGraphicsHandler* m_sgh;
 
};

#endif

