
/* CellWriterForm declaration 

Copyright (C) 2009 Roberto Navone <ron@object65.it>
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; version 2
of the License.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
*/

#ifndef CELLWRITERFORMH
#define CELLWRITERFORMH

#include "GraphicsHandler.h"
#include "TimersHandler.h"
#include "ButtonsHandler.h"
#include "CheckBoxesHandler.h"
#include "CellWriterEngine.h"

class CellWriterForm
{
public:

  /*
   * Constructor
   */
  CellWriterForm(CellWriterEngine* eng, MainGraphicsHandler* mgh, SubGraphicsHandler* sgh, TimersHandler* th);
  
  /*
   * Destructor
   */
  virtual ~CellWriterForm();
  
  /*
   * Run results
   */
  enum RunResult
  {
    CHANGEMODE,
    EXIT,
  };
  
  /*
   * Run the form
   */
  RunResult Run();
  
protected:

  /*
   * Initialize the form 
   */
  virtual void InitForm() = 0;
  
  /*
   * Check if requested command is form's exit command
   */
  virtual bool IsExitCommand(int cmd) = 0;
  
  /*
   * Check if requested command is form's change mode command
   */
  virtual bool IsChangeModeCommand(int cmd) = 0;
  
  /*
   * Return the timeout command code
   */
  virtual int GetTimeoutCommand() = 0;
  
  /*
   * Handle the requested command
   */
  virtual void HandleCommand(int cmd) = 0;
  
  /*
   * Get timeout for symbol sampling 
   */
  virtual int GetSamplingTimeout() = 0;
  
  /*
   * Draw a character cell at requested position
   */
  void DrawCell(GraphicsHandler* gh, int x, int y);
  
  /*
   * Draw a character cell at requested position
   */
  void DrawCell(GraphicsHandler* gh, int x, int y, int c, unsigned short color);
  
  /*
   * Check if requested point is into cell area
   */
  virtual bool IsInCellArea(int x, int y) = 0;
  
  /*
   * Handle a button press event in cell area
   */
  virtual void HandleButtonPressEvent(int x, int y) = 0;
  
  /*
   * Handle a button release event in cell area
   */
  virtual void HandleButtonReleaseEvent() = 0;
  
  /*
   * Handle a motion notify event in cell area
   */
  virtual void HandleMotionNotifyEvent(int x, int y) = 0;
  
  /*
   * Handle a finish timeout event in cell area
   */
  virtual void HandleFinishTimeout() = 0;
  
  /*
   * Cell writer engine
   */
  CellWriterEngine* m_eng;
  
  /*
   * Graphics handlers
   */
  MainGraphicsHandler* m_mgh;
  SubGraphicsHandler* m_sgh;
  
  /*
   * Buttons handler
   */
  ButtonsHandler* m_bh;
  
  /*
   * CheckBoxes handler
   */
  CheckBoxesHandler* m_cbxh;
  
private:

  /*
   * Get a command
   */
  int GetCommand();
 
  /*
   * Wait touch release
   */
  void WaitTouchRelease();
  
  /*
   * Timers handler
   */
  TimersHandler* m_th;
  
};

#ifdef CELLSCRDEFS

#define X_CELL      108
#define Y_UPCELL    40
#define Y_DNCELL    90
#define W_CELL      40
#define H_CELL      70
#define H_UPCELL    50
#define H_DNCELL    20

#endif
#endif
