
/* CellWriterTrainForm implementation 

Copyright (C) 2009 Roberto Navone <ron@object65.it>
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; version 2
of the License.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
*/

#include <nds.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "colors.h"
#include "LiteralHandler.h"
#define TRAINSCRDEFS
#include "CellWriterTrainForm.h"
//#define ALLOWSAVE

static bool sRowCacheInit = false;
static void* sRowCache[NSYMROWS];

/*
 * Constructor
 */
CellWriterTrainForm::CellWriterTrainForm(CellWriterEngine* eng, MainGraphicsHandler* mgh, SubGraphicsHandler* sgh, TimersHandler* th) : CellWriterForm(eng, mgh, sgh, th)
{
  if (!sRowCacheInit)
  {
    for (int i=0; i<NSYMROWS; i++)
      sRowCache[i] = 0;
      
    sRowCacheInit = true;
  }
}

/*
 * Initialize the form 
 */
void CellWriterTrainForm::InitForm()
{
  /* Title */
  
  m_mgh->DrawTextTrueType(GraphicsHandler::TTF_tuffybold, LIT_TRAINMODE, X_TITLE, Y_TITLE, H_TITLE, TXT_COLOR, SCR_BACK);
  
  /* Sample cells */
  
  for (int i=0; i<NSAMCELLS; i++)
    DrawCell(m_mgh, (X_SAMFIRSTCELL + i * W_SAMCELL), Y_SAMCELL);
    
  /* Selections cells & grid touch buttons */
  
  for (int i=0; i<NVSYMCELLS; i++)
    DrawSymCellRow(i, i);
      
  m_nCurGridRow = 0;    // first row
  m_nCurGridCell = -1;   // no grid cell selected
    
  /* Add button area for cell selection */
  
  for (int i=0; i<NVSYMCELLS; i++)
    for (int j=0; j<NHSYMCELLS; j++)
      m_bh->AddButtonArea(((int)IDC_SELECTGRIDBASE + i * NHSYMCELLS + j), (X_SELFIRSTCELL + j * W_SELCELL), (Y_SELFIRSTCELL + i * H_SELCELL), W_SYMCELL, (H_UPSYMCELL + H_DNSYMCELL));
    
  /* Grid navigation buttons */
  
  static char uparrow[] = { UPARROW, '\0' };
  static char dnarrow[] = { DNARROW, '\0' };
  
  m_prevButton = m_bh->AddButton((int)IDC_PREVROW, X_NEXTPREVARW, Y_PREVARW, W_NEXTPREVARW, H_NEXTPREVARW, uparrow);
  m_nextButton = m_bh->AddButton((int)IDC_NEXTROW, X_NEXTPREVARW, Y_NEXTARW, W_NEXTPREVARW, H_NEXTPREVARW, dnarrow);
  
  m_sgh->DrawRectangle(X_NEXTPREVARW, (Y_PREVARW + H_NEXTPREVARW + 1), (X_NEXTPREVARW + W_NEXTPREVARW - 1), (Y_NEXTARW - 1), SCRL_BACK, true);
  
  m_prevButton->Disable();
  
  /* Symbol selection literal */
  
  m_sgh->DrawTextTrueType(GraphicsHandler::TTF_goulong, LIT_SELECTSYMBOL, X_SELSYM, Y_SELSYM, H_SELSYM, TXT_COLOR, SCR_BACK);
  
  /* Edit button */
  
  m_editButton = m_bh->AddButton((int)IDC_EDIT, X_BOTBTN1, Y_BOTBTNS, W_BOTBTNS, H_BOTBTNS, LIT_EDITBUTTON);
  
  m_editButton->Disable();
  
  /* Save button */

#ifdef ALLOWSAVE

  m_saveButton = m_bh->AddButton((int)IDC_SAVE, X_BOTBTN2, Y_BOTBTNS, W_BOTBTNS, H_BOTBTNS, LIT_SAVEBUTTON);
  
  //m_saveButton->Disable();
  
#endif
  
  /* Switch mode button */
  
  m_bh->AddButton((int)IDC_WRITE, X_BOTBTN3, Y_BOTBTNS, W_BOTBTNS, H_BOTBTNS, LIT_WRITEBUTTON);
  
  /* Not in edit mode */
  
  m_inEditMode = false;
  
  /* Logo! */
  
  m_mgh->DrawTextTrueType(GraphicsHandler::TTF_goulong, LIT_WEBSITE, -1, 190, 11, 0xffff);
}

/*
 * Check if requested command is form's exit command
 */
bool CellWriterTrainForm::IsExitCommand(int cmd)
{
  return (false);
}

/*
 * Check if requested command is form's change mode command
 */
bool CellWriterTrainForm::IsChangeModeCommand(int cmd)
{
  return (cmd == (int)IDC_WRITE);
}

/*
 * Return the timeout command code
 */
int CellWriterTrainForm::GetTimeoutCommand()
{
  return ((int)IDC_TIMEOUT);
} 

/*
 * Handle the requested command
 */
void CellWriterTrainForm::HandleCommand(int cmd)
{
  /* Handle the command */
  
  switch (cmd)
  {
    case IDC_PREVROW:   // previous row on grid
    
      DeselectCurGridCell();
      
      if (ScrollUpGrid())
        m_prevButton->Disable();
        
      if (!m_nextButton->	IsEnabled())
        m_nextButton->Enable();
      
      break;
      
    case IDC_NEXTROW:   // next row on grid
    
      DeselectCurGridCell();
      
      if (ScrollDownGrid())
        m_nextButton->Disable();
        
      if (!m_prevButton->	IsEnabled())
        m_prevButton->Enable();
      
      break;
      
    case IDC_EDIT:      // enter edit mode
    
      EnterEditMode();
      
      break;
      
    case IDC_ENDEDIT:   // leave edit mode
    
      LeaveEditMode();
      
      break;
      
    case IDC_RESET:     // reset current char samples
      ResetSamplesForCurrentChar();
      break;
      
    case IDC_TIMEOUT:     // char trained
      HandleTrainedChar();
      break;
      
    case IDC_SAVE:    // save samples
      SaveSamples();
      
    default:
    
      if (cmd >= IDC_SELECTGRIDBASE  &&  cmd < (IDC_SELECTGRIDBASE + NHSYMCELLS * NVSYMCELLS))
      {
        /* Grid cell selected */
        
        SelectGridCell(cmd - IDC_SELECTGRIDBASE);
      }
      
      break;
  }
}

/*
 * Handle a button press event in cell area
 */
void CellWriterTrainForm::HandleButtonPressEvent(int x, int y)
{
  if (!m_eng->IsInTrainingMode())
    m_eng->BeginTrainingMode(m_ch, X_CELL, Y_UPCELL, W_CELL, H_CELL, CELL_DRAW);
    
  m_eng->NotifyButtonPressEvent(x - X_CELL, y - Y_UPCELL);
}

/*
 * Handle a button release event in cell area
 */
void CellWriterTrainForm::HandleButtonReleaseEvent()
{
  m_eng->NotifyButtonReleaseEvent();
}

/*
 * Handle a motion notify event in cell area
 */
void CellWriterTrainForm::HandleMotionNotifyEvent(int x, int y)
{
  m_eng->NotifyMotionEvent(x - X_CELL, y - Y_UPCELL);
}

/*
 * Handle a finish timeout event in cell area
 */
void CellWriterTrainForm::HandleFinishTimeout()
{
  m_eng->EndTrainingMode();
}
  
/*
 * Reset samples for current char
 */
void CellWriterTrainForm::ResetSamplesForCurrentChar()
{
  /* Reset data for current char */
  
  m_eng->ResetSamples(m_ch);
  
  /* Set current char again to reload values and adjust screen */
  
  SetCurrentChar(m_ch);
  
  /* Disable clear button */
  
  m_resetButton->Disable();
}

/*
 * Set the requested current character
 */
void CellWriterTrainForm::SetCurrentChar(int ch)
{
  /* Set current char */
  
  m_ch = ch;
  
  m_nSamples = ch == -1 ? -1 : m_eng->GetNumSamples(ch);
  
  /*
  m_nSamples = m_eng->GetNumSamples(ch);
  
  m_curSample = -1;
  */
  
  /* Draw samples */
  
  if (m_nSamples > 0)
  {
    for (int i=0; i<m_nSamples; i++)
    {
      DrawCell(m_mgh, (X_SAMFIRSTCELL + i * W_SAMCELL), Y_SAMCELL, m_ch, CELL_DRAW);
      
      m_eng->DrawSample(m_ch, i, true, (X_SAMFIRSTCELL + i * W_SAMCELL), Y_SAMCELL, W_CELL, H_CELL, CELL_INK);
    }
  }
  else
  {
    for (int i=0; i<NSAMCELLS; i++)
      DrawCell(m_mgh, (X_SAMFIRSTCELL + i * W_SAMCELL), Y_SAMCELL);
  }
  
  /* Adjust the screen */
  
  ShowCurrentChar();
  ShowNumSamples();
}

/*
 * Show current character
 */
void CellWriterTrainForm::ShowCurrentChar()
{
  m_mgh->DrawRectangle(X_CURCHAR, (Y_CURCHAR - H_CURCHAR), (X_CURCHAR + W_CURCHAR), (Y_CURCHAR + 3), SCR_BACK, true);

  if (m_ch != -1)
  {
    char buff[12];
  
    sprintf(buff, LIT_CURSYMBOL, m_ch);
  
    m_mgh->DrawTextTrueType(GraphicsHandler::TTF_goulong, buff, X_CURCHAR, Y_CURCHAR, H_CURCHAR, TXT_COLOR, SCR_BACK);
  }
}

/*
 * Show the number of samples
 */
void CellWriterTrainForm::ShowNumSamples()
{
  m_mgh->DrawRectangle(X_NSAMP, (Y_NSAMP - H_NSAMP), (X_NSAMP + W_NSAMP), (Y_NSAMP + 3), SCR_BACK, true);

  if (m_nSamples != -1)
  {
    char buff[12];
  
    if (m_nSamples == 1)
      strcpy(buff, LIT_1SAMPLE);
    else
      sprintf(buff, LIT_NSAMPLES, m_nSamples);
  
    m_mgh->DrawTextTrueType(GraphicsHandler::TTF_goulong, buff, X_NSAMP, Y_NSAMP, H_NSAMP, TXT_COLOR, SCR_BACK);
  }
}

/*
 * Show the number of current sample
 */
void CellWriterTrainForm::ShowCurSampleNumber()
{
  char buff[12];
  
  if (m_curSample >= 0)
    sprintf(buff, "sample %d", m_curSample+1);
  else
    strcpy(buff, "         ");
  
  m_sgh->DrawText(buff, X_NSAMP, Y_CSAMP, TXT_COLOR, true, SCR_BACK);
}

/*
 * Draw current sample in cell area 
 */
void CellWriterTrainForm::DrawCurSample()
{
  //m_eng->DrawSample(m_ch, m_curSample, X_CELL, Y_UPCELL, W_CELL, H_CELL, CELL_DRAW);
}

/*
 * Handle the trained char, after training input timeout
 */
void CellWriterTrainForm::HandleTrainedChar()
{
  /* All is done, restore the screen */
  
  SetCurrentChar(m_ch);
  
  DrawCell(m_sgh, X_CELL, Y_UPCELL, m_ch, CELL_EDIT);
  
  m_resetButton->Enable();  
}

/*
 * Save samples
 */
void CellWriterTrainForm::SaveSamples()
{
  m_eng->SaveSamples("fat:/cwsamples.dat");
  
  //m_saveButton->Disable();
}

/*
 * Draw an empty symbol cell at requested position
 */
void CellWriterTrainForm::DrawSymCell(int x, int y)
{
  m_sgh->DrawRectangle(x, y, (x + W_SYMCELL), (y + H_UPSYMCELL), CELL_UPBACK, true);
  m_sgh->DrawRectangle(x, (y + H_UPSYMCELL), (x + W_SYMCELL), (y + H_UPSYMCELL + H_DNSYMCELL), CELL_DNBACK, true);
}

/*
 * Draw a symbol cell at requested position
 */
void CellWriterTrainForm::DrawSymCell(int x, int y, int c)
{
  /* First the cell */
  
  DrawSymCell(x, y);
  
  /* Then the content */
  
  int nCustom = -1;
  
  if (c >= 0x7F)
  {
    nCustom = c - 0x7F;
    
    c = '?';
  }
  
  char txt[4];
  txt[0] = c < 0x7F ? c : '?';
  txt[1] = '\0';
  
  int h = (H_UPSYMCELL + H_DNSYMCELL - 3);
  
  while (h > 0)
  {
    int w = m_sgh->GetTextTrueTypeWidth(GraphicsHandler::TTF_tuffybold, txt, h);
    
    if (w > W_SYMCELL)
    {
      h--;
      continue;
    }
  
    m_sgh->DrawTextTrueType(GraphicsHandler::TTF_tuffybold, txt, (x + (W_SYMCELL - w) / 2), (y + H_UPSYMCELL), h, TXT_COLOR);
    
    break;
  }
  
  /* Draw custom symbol number */
  
  if (nCustom >= 0)
  {
    sprintf(txt, "%d", nCustom);
    
    h = H_DNSYMCELL;
    
    int w = m_sgh->GetTextTrueTypeWidth(GraphicsHandler::TTF_tuffybold, txt, h);
    
    m_sgh->DrawTextTrueType(GraphicsHandler::TTF_tuffybold, txt, (x + (W_SYMCELL - w) / 2), (y + H_UPSYMCELL + H_DNSYMCELL), h, TXT_COLOR);
  }
}
  
/*
 * Scroll grid one line up
 */
bool CellWriterTrainForm::ScrollUpGrid()
{
  int w = NHSYMCELLS * W_SELCELL;
  int h = (NVSYMCELLS - 1) * H_SELCELL;
  
  void *p = m_sgh->SaveBackground(X_SELFIRSTCELL, Y_SELFIRSTCELL, w, h);
  
  m_sgh->RestoreBackground(p, X_SELFIRSTCELL, (Y_SELFIRSTCELL + H_SELCELL), w, h);
  
  m_nCurGridRow--;
  
  DrawSymCellRow(0, m_nCurGridRow);
  
  return (m_nCurGridRow == 0);
}

/*
 * Scroll grid one line down
 */
bool CellWriterTrainForm::ScrollDownGrid()
{
  int w = NHSYMCELLS * W_SELCELL;
  int h = (NVSYMCELLS - 1) * H_SELCELL;
  
  void *p = m_sgh->SaveBackground(X_SELFIRSTCELL, (Y_SELFIRSTCELL + H_SELCELL), w, h);
  
  m_sgh->RestoreBackground(p, X_SELFIRSTCELL, Y_SELFIRSTCELL, w, h);
  
  m_nCurGridRow++;
  
  DrawSymCellRow((NVSYMCELLS - 1), (m_nCurGridRow + NVSYMCELLS - 1));
  
  return (m_nCurGridRow == (NSYMROWS - NVSYMCELLS));
}

/*
 * select the requested grid cell
 */
void CellWriterTrainForm::SelectGridCell(int nGridCell)
{
  DeselectCurGridCell();

  DrawGridCellSelector(nGridCell, GRIDCELL);
  
  m_nCurGridCell = nGridCell;
  
  if (!m_editButton->IsEnabled())
    m_editButton->Enable();
    
  SetCurrentChar(m_nCurGridRow * NHSYMCELLS + nGridCell + 0x21);
}

/*
 * Deselect the current grid cell
 */
void CellWriterTrainForm::DeselectCurGridCell()
{
  if (m_nCurGridCell == -1)
    return;
    
  DrawGridCellSelector(m_nCurGridCell, SCR_BACK);
  
  m_nCurGridCell = -1;
  
  m_editButton->Disable();

  SetCurrentChar(-1);
}

/*
 * Draw a grid cell selector around the required cell
 */
void CellWriterTrainForm::DrawGridCellSelector(int nGridCell, unsigned short color)
{
  int x = X_SELFIRSTCELL + (nGridCell % NHSYMCELLS) * W_SELCELL - 1;
  int y = Y_SELFIRSTCELL + (nGridCell / NHSYMCELLS) * H_SELCELL - 1;
  
  m_sgh->DrawRectangle(x, y, (x + W_SYMCELL + 2), (y + H_UPSYMCELL + H_DNSYMCELL + 1), color, false);
}

/*
 * Enter in edit mode
 */
void CellWriterTrainForm::EnterEditMode()
{
  m_inEditMode = true;
  
  /* Disable all buttons */
  
  for (int i = 0; i<(NHSYMCELLS * NVSYMCELLS); i++)
    m_bh->GetButton(IDC_SELECTGRIDBASE + i)->Disable();
  
  m_prevButton->Disable();
  m_nextButton->Disable();
  
  m_bh->GetButton(IDC_WRITE)->Disable();
#ifdef ALLOWSAVE  
  m_bh->GetButton(IDC_SAVE)->Disable();
#endif  
  m_bh->GetButton(IDC_EDIT)->Disable();
  
  /* Grab window */
  
  m_savedBackground = m_sgh->SaveBackground(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
  
  /* Clear background */
  
  m_sgh->ClearScreen(SCR_BACK);
  
  /* Draw cella area */
  
  DrawCell(m_sgh, X_CELL, Y_UPCELL, m_ch, CELL_EDIT);
  
  /* Add buttons */
  
  m_resetButton = m_bh->AddButton((int)IDC_RESET, X_BOTBTN1, Y_BOTBTNS, W_BOTBTNS, H_BOTBTNS, LIT_RESETBUTTON);
  
  if (m_nSamples == 0)
    m_resetButton->Disable();
  
  m_bh->AddButton((int)IDC_ENDEDIT, X_BOTBTN3, Y_BOTBTNS, W_BOTBTNS, H_BOTBTNS, LIT_ENDEDITBUTTON);

  /* Symbol edit literal */
  
  m_sgh->DrawTextTrueType(GraphicsHandler::TTF_goulong, LIT_EDITSYMBOL, X_EDTSYM, Y_EDTSYM, H_EDTSYM, TXT_COLOR, SCR_BACK);
}

/*
 * Leave edit mode
 */
void CellWriterTrainForm::LeaveEditMode()
{
  m_inEditMode = false;
  
  /* Remove buttons */
  
  m_bh->RemoveButton(IDC_RESET);
  m_bh->RemoveButton(IDC_ENDEDIT);
  
  /* Restore window */
  
  m_sgh->RestoreBackground(m_savedBackground, 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);

  /* Enable all buttons */
  
  for (int i = 0; i<(NHSYMCELLS * NVSYMCELLS); i++)
    m_bh->GetButton(IDC_SELECTGRIDBASE + i)->Enable();
  
  if (m_nCurGridRow > 0)
    m_prevButton->Enable();
  
  if (m_nCurGridRow < (NSYMROWS - NVSYMCELLS))
    m_nextButton->Enable();
  
  m_bh->GetButton(IDC_WRITE)->Enable();
#ifdef ALLOWSAVE  
  m_bh->GetButton(IDC_SAVE)->Enable();
#endif  
  m_bh->GetButton(IDC_EDIT)->Enable();
}

/*
 * Draw the requested symbol row
 */
void CellWriterTrainForm::DrawSymCellRow(int nRow, int nSymRow)
{
  if (sRowCache[nSymRow] == 0)
  {
    /* Draw */
    
    for (int i=0; i<NHSYMCELLS; i++)
      DrawSymCell((X_SELFIRSTCELL + i * W_SELCELL), (Y_SELFIRSTCELL + nRow * H_SELCELL), (0x21 + nSymRow * NHSYMCELLS + i));
      
    /* Store in cache */
    
    sRowCache[nSymRow] = m_sgh->SaveBackground(X_SELFIRSTCELL, (Y_SELFIRSTCELL + nRow * H_SELCELL), (NHSYMCELLS * W_SELCELL), H_SELCELL);
  }
  else
  {
    /* Restore from cache */
    
    m_sgh->WriteBackground(sRowCache[nSymRow], X_SELFIRSTCELL, (Y_SELFIRSTCELL + nRow * H_SELCELL), (NHSYMCELLS * W_SELCELL), H_SELCELL);
  }
}

/*
 * Check if requested point is into cell area
 */
bool CellWriterTrainForm::IsInCellArea(int x, int y)
{
  if (!m_inEditMode)
    return (false);
    
  return ((x >= X_CELL  &&  x <= (X_CELL + W_CELL))  &&  (y >= Y_UPCELL  &&  y <= (Y_UPCELL + H_CELL)));
}
