
/* CellWriterTrainForm declaration 

Copyright (C) 2009 Roberto Navone <ron@object65.it>
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; version 2
of the License.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
*/

#ifndef CELLWRITERTRAINFORMH
#define CELLWRITERTRAINFORMH

#include "CellWriterForm.h"

class CellWriterTrainForm : public CellWriterForm
{
public:

  /*
   * Constructor
   */
  CellWriterTrainForm(CellWriterEngine* eng, MainGraphicsHandler* mgh, SubGraphicsHandler* sgh, TimersHandler* th);
  
protected:

  /*
   * Initialize the form 
   */
  void InitForm();
  
  /*
   * Check if requested command is form's exit command
   */
  bool IsExitCommand(int cmd);
  
  /*
   * Check if requested command is form's change mode command
   */
  bool IsChangeModeCommand(int cmd);
  
  /*
   * Return the timeout command code
   */
  int GetTimeoutCommand();
  
  /*
   * Handle the requested command
   */
  void HandleCommand(int cmd);
  
  /*
   * Handle a button press event in cell area
   */
  void HandleButtonPressEvent(int x, int y);
  
  /*
   * Handle a button release event in cell area
   */
  void HandleButtonReleaseEvent();
  
  /*
   * Handle a motion notify event in cell area
   */
  void HandleMotionNotifyEvent(int x, int y);
  
  /*
   * Handle a finish timeout event in cell area
   */
  void HandleFinishTimeout();
  
  /*
   * Check if requested point is into cell area
   */
  bool IsInCellArea(int x, int y);
  
  /*
   * Get timeout for symbol sampling 
   */
  int GetSamplingTimeout() { return (48); } // 800 msec
  
private:

  /*
   * Set the requested current character
   */
  void SetCurrentChar(int c);

  /*
   * Reset samples for current char
   */
  void ResetSamplesForCurrentChar();

  /*
   * Show current character
   */
  void ShowCurrentChar();

  /*
   * Show the number of samples for the current char
   */
  void ShowNumSamples();

  /*
   * Show the number of current sample
   */
  void ShowCurSampleNumber();
  
  /*
   * Draw current sample in cell area 
   */
  void DrawCurSample();

  /*
   * Handle the trained char, after training input timeout
   */
  void HandleTrainedChar();
  
  /*
   * Save samples
   */
  void SaveSamples();
  
  /*
   * Draw an empty symbol cell at requested position
   */
  void DrawSymCell(int x, int y);

  /*
   * Draw a symbol cell at requested position
   */
  void DrawSymCell(int x, int y, int c);

  /*
   * Clear the symbol cell at requested position
   */
  void ClearSymCell(int x, int y) { DrawSymCell(x, y); }
  
  /*
   * Scroll grid one line up
   */
  bool ScrollUpGrid();
  
  /*
   * Scroll grid one line down
   */
  bool ScrollDownGrid();
  
  /*
   * select the requested grid cell
   */
  void SelectGridCell(int nGridCell);
  
  /*
   * Deselect the current grid cell
   */
  void DeselectCurGridCell();
  
  /*
   * Draw a grid cell selector around the required cell
   */
  void DrawGridCellSelector(int nGridCell, unsigned short color);
  
  /*
   * Enter in edit mode
   */
  void EnterEditMode();
  
  /*
   * Leave edit mode
   */
  void LeaveEditMode();
  
  /*
   * Draw the requested symbol row
   */
  void DrawSymCellRow(int nRow, int nSymRow);
    
  /*
   * Commands
   */
  enum Command
  {
    IDC_SELECTGRIDBASE = 0,
    IDC_WRITE = 100,
    IDC_EDIT,
    IDC_ENDEDIT,
    IDC_SAVE,
    IDC_RESET,
    IDC_TIMEOUT,
    IDC_PREVROW,
    IDC_NEXTROW,
  };
  
  /*
   * Current train char
   */
  int m_ch;
  
  /*
   * Number of samples and current sample (0 based) for the current train char
   */
  int m_nSamples, m_curSample;
  
  /*
   * Current grid row
   */
  int m_nCurGridRow;
  
  /*
   * Current grid cell
   */
  int m_nCurGridCell;
  
  /*
   * Buttons for grid navigation
   */
  Button *m_prevButton, *m_nextButton;
  
  /*
   * Command buttons
   */
  Button *m_editButton, *m_saveButton, *m_resetButton;
  
  /*
   * Pointer to saved background switching from selection to edit
   */
  void *m_savedBackground;
  
  /*
   * Edit mode flag
   */
  bool m_inEditMode;
  
};

#ifdef TRAINSCRDEFS

#define X_TITLE   4
#define Y_TITLE   22
#define H_TITLE   20

#define NSAMCELLS  6

#define X_SAMFIRSTCELL   4
#define W_SAMCELL        42
#define Y_SAMCELL        75

#define NHSYMCELLS  11
#define NVSYMCELLS  4

#define X_SELFIRSTCELL  2
#define W_SELCELL       22
#define Y_SELFIRSTCELL  2
#define H_SELCELL       33

#define W_SYMCELL      20
#define H_SYMCELL      35
#define H_UPSYMCELL    23
#define H_DNSYMCELL    9

#define NSYMROWS    10

#define X_NEXTPREVARW   245
#define Y_PREVARW       2
#define Y_NEXTARW       123
#define H_NEXTPREVARW   10
#define W_NEXTPREVARW   10

#define X_BOTBTN1   4
#define X_BOTBTN2   88
#define X_BOTBTN3   172
#define Y_BOTBTNS   160
#define W_BOTBTNS   80
#define H_BOTBTNS   30

#define X_CELL      108
#define Y_UPCELL    40
#define Y_DNCELL    90
#define W_CELL      40
#define H_CELL      70
#define H_UPCELL    50
#define H_DNCELL    20

#define X_PNBTNS    25
#define Y_PVBTN     45
#define Y_NXBTN     85
#define W_PNBTNS    60
#define H_PNBTNS    20

#define X_RSBTN     170
#define Y_RSBTN     65
#define W_RSBTN     60
#define H_RSBTN     20

#define X_CURCHAR   5
#define Y_CURCHAR   45
#define H_CURCHAR   12
#define W_CURCHAR   200

#define X_NSAMP     5
#define Y_NSAMP     65
#define H_NSAMP     12
#define W_NSAMP     200

#define Y_SHBTN     90
#define Y_CSAMP     140
#define Y_PSBTN     115
#define Y_NSBTN     160

#define X_SELSYM    5
#define Y_SELSYM    152
#define H_SELSYM    12

#define X_EDTSYM    5
#define Y_EDTSYM    152
#define H_EDTSYM    12

#endif
#endif
