
/* CellWriterTryForm implementation 

Copyright (C) 2009 Roberto Navone <ron@object65.it>
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; version 2
of the License.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
*/

#include <nds.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "colors.h"
#include "LiteralHandler.h"
#define TRYSCRDEFS
#include "CellWriterWriteForm.h"

/*
 * Constructor
 */
CellWriterWriteForm::CellWriterWriteForm(CellWriterEngine* eng, MainGraphicsHandler* mgh, SubGraphicsHandler* sgh, TimersHandler* th) : CellWriterForm(eng, mgh, sgh, th)
{
  m_flgCellArea = new bool[NWRCELLS];
}

/*
 * Initialize the form 
 */
void CellWriterWriteForm::InitForm()
{
  /* Title */
  
  m_mgh->DrawTextTrueType(GraphicsHandler::TTF_tuffybold, LIT_WRITEMODE, X_TITLE, Y_TITLE, H_TITLE, TXT_COLOR, SCR_BACK);
  
  /* Cells */
  
  for (int i=0; i<NWRCELLS; i++)
  {
    int x = X_WRFIRSTCELL + i * W_WRCELL;
    
    DrawCell(m_mgh, x, Y_WRCELLRES);
    DrawCell(m_sgh, x, Y_WRCELLINP);
  }
  
  /* Recognition result placeholders 
  
  for (int i=0; i<NWRCELLS; i++)
    for (int j=0; j<m_eng->GetMaxRecognitionAlts(); j++)
      ShowRecognitionResult(i, j, 'X', 100);
      
  */
  
  /* CheckBoxes */

  m_sgh->DrawTextTrueType(GraphicsHandler::TTF_goulong, LIT_ENABLEDSYMBOLS, X_ENABLEDLIT, Y_ENABLEDLIT, H_ENABLEDLIT, TXT_COLOR, SCR_BACK);
  
  m_cbxh->AddCheckBox((int)IDC_CBXNUMBERS, X_CBXNUMBERS, Y_CBXNUMBERS, false);
  m_cbxh->AddCheckBox((int)IDC_CBXCAPLETTERS, X_CBXCAPLETTERS, Y_CBXCAPLETTERS, true);
  m_cbxh->AddCheckBox((int)IDC_CBXLETTERS, X_CBXLETTERS, Y_CBXLETTERS, false);

  m_cbxh->AddCheckBox((int)IDC_CBXSYMBOLS, X_CBXSYMBOLS, Y_CBXSYMBOLS, false);
  m_cbxh->AddCheckBox((int)IDC_CBXCUSTOM, X_CBXCUSTOM, Y_CBXCUSTOM, false);

  m_sgh->DrawTextTrueType(GraphicsHandler::TTF_goulong, LIT_ENABNUMBERS, X_NUMBERS, Y_NUMBERS, H_ENABLEDLIT, TXT_COLOR, SCR_BACK);
  m_sgh->DrawTextTrueType(GraphicsHandler::TTF_goulong, LIT_ENABCAPLETTERS, X_CAPLETTERS, Y_CAPLETTERS, H_ENABLEDLIT, TXT_COLOR, SCR_BACK);
  m_sgh->DrawTextTrueType(GraphicsHandler::TTF_goulong, LIT_ENABLETTERS, X_LETTERS, Y_LETTERS, H_ENABLEDLIT, TXT_COLOR, SCR_BACK);
  m_sgh->DrawTextTrueType(GraphicsHandler::TTF_goulong, LIT_ENABSYMBOLS, X_SYMBOLS, Y_SYMBOLS, H_ENABLEDLIT, TXT_COLOR, SCR_BACK);
  m_sgh->DrawTextTrueType(GraphicsHandler::TTF_goulong, LIT_ENABCUSTOM, X_CUSTOM, Y_CUSTOM, H_ENABLEDLIT, TXT_COLOR, SCR_BACK);
  
  /* Switch mode button */
  
  m_bh->AddButton((int)IDC_TRAIN, X_BOTBTN3, Y_BOTBTNS, W_BOTBTNS, H_BOTBTNS, LIT_TRAINBUTTON);
  
  /* Delete button */
  
  m_deleteButton = m_bh->AddButton((int)IDC_DELETE, X_BOTBTN1, Y_BOTBTNS, W_BOTBTNS, H_BOTBTNS, LIT_DELETEBUTTON);
  
  m_deleteButton->Disable();
  
  /* No cell area selected */
  
  m_curCellArea = -1;
  
  for (int i=0; i<NWRCELLS ; i++)
    m_flgCellArea[i] = false;
    
  /* All disabled except capital letters */
  
  EnableNumbers(false);
  EnableCapLetters(true);
  EnableLetters(false);
  EnableSymbols(false);
  EnableCustoms(false);

  /* Logo! */
  
  m_mgh->DrawTextTrueType(GraphicsHandler::TTF_goulong, LIT_WEBSITE, -1, 190, 11, 0xffff);
}

/*
 * Check if requested command is form's exit command
 */
bool CellWriterWriteForm::IsExitCommand(int cmd)
{
  return (false);
}

/*
 * Check if requested command is form's change mode command
 */
bool CellWriterWriteForm::IsChangeModeCommand(int cmd)
{
  return (cmd == (int)IDC_TRAIN);
}

/*
 * Return the timeout command code
 */
int CellWriterWriteForm::GetTimeoutCommand()
{
  return ((int)IDC_TIMEOUT);
} 

/*
 * Handle the requested command
 */
void CellWriterWriteForm::HandleCommand(int cmd)
{
  switch (cmd)
  {
    case IDC_DELETE:      // delete draw
      PerformDrawDelete();
      break;
      
    case IDC_TIMEOUT:     // char drawed
      HandleRecognizedChar();
      break;
      
    case IDC_CBXNUMBERS:    // numbers checkbox toggled
      EnableNumbers(m_cbxh->GetCheckBox(IDC_CBXNUMBERS)->IsChecked());
      break;
      
    case IDC_CBXCAPLETTERS: // capital letters checkbox toggled
      EnableCapLetters(m_cbxh->GetCheckBox(IDC_CBXCAPLETTERS)->IsChecked());
      break;
      
    case IDC_CBXLETTERS:    // letters checkbox toggled
      EnableLetters(m_cbxh->GetCheckBox(IDC_CBXLETTERS)->IsChecked());
      break;
      
    case IDC_CBXSYMBOLS:    // symbols checkbox toggled
      EnableSymbols(m_cbxh->GetCheckBox(IDC_CBXSYMBOLS)->IsChecked());
      break;
      
    case IDC_CBXCUSTOM:     // custom symbols checkbox toggled
      EnableCustoms(m_cbxh->GetCheckBox(IDC_CBXCUSTOM)->IsChecked());
      break;
      
    default:
      break;
  }
}

/*
 * Handle a button press event in cell area
 */
void CellWriterWriteForm::HandleButtonPressEvent(int x, int y)
{
  int xc = X_WRFIRSTCELL + m_curCellArea * W_WRCELL;

  if (!m_eng->IsInRecognizeMode())
    m_eng->BeginRecognizeMode(xc, Y_WRCELLINP, W_CELL, H_CELL, CELL_DRAW);
    
  m_eng->NotifyButtonPressEvent(x - xc, y - Y_WRCELLINP);
}

/*
 * Handle a button release event in cell area
 */
void CellWriterWriteForm::HandleButtonReleaseEvent()
{
  m_eng->NotifyButtonReleaseEvent();
}

/*
 * Handle a motion notify event in cell area
 */
void CellWriterWriteForm::HandleMotionNotifyEvent(int x, int y)
{
  int xc = X_WRFIRSTCELL + m_curCellArea * W_WRCELL;

  m_eng->NotifyMotionEvent(x - xc, y - Y_WRCELLINP);
}

/*
 * Handle a finish timeout event in cell area
 */
void CellWriterWriteForm::HandleFinishTimeout()
{
  m_ch = m_eng->EndRecognizeMode();
}
 

/*
 * Show recognition result
 */
void CellWriterWriteForm::ShowRecognitionResult(int nchar, int nalt, char c, int pct)
{
  char buff[10];
  
  if (pct >= 0)
    sprintf(buff, "%c %3d%%", c, pct);
  else
    strcpy(buff, "      ");
  
  int x = X_WRFIRSTCELL + W_WRCELL * nchar;
  int y = Y_RECRES + H_RECSPC * nalt;
  
  m_mgh->DrawTextTrueType(GraphicsHandler::TTF_goulong, buff, x, y, H_RECRES, TXT_COLOR, SCR_BACK);
}

/*
 * Show recognition time
 */
void CellWriterWriteForm::ShowRecognitionTime(int msec)
{
  char buff[10];
  
  if (msec >= 0)
    sprintf(buff, "%d msec", msec);
  else
    strcpy(buff, "         ");
  
  m_sgh->DrawText(buff, X_RECTM, Y_RECRES + (m_eng->GetMaxRecognitionAlts() + 1) * (m_sgh->GetTextHeight() + 1), TXT_COLOR, true, SCR_BACK);
}

/*
 * Clear all recognition results
 */
void CellWriterWriteForm::ClearRecognitionResults()
{
  /*
  for (int i=0; i<m_eng->GetMaxRecognitionAlts(); i++)
    ShowRecognitionResult(i, ' ', -1);
    
  ShowRecognitionTime(-1);
  */
}

/*
 * Delete current draw
 */
void CellWriterWriteForm::PerformDrawDelete()
{
  for (int i=0; i<NWRCELLS; i++)
  {
    int x = X_WRFIRSTCELL + i * W_WRCELL;
    
    DrawCell(m_mgh, x, Y_WRCELLRES);
    DrawCell(m_sgh, x, Y_WRCELLINP);
  }
  
  m_curCellArea = -1;
  
  for (int i=0; i<NWRCELLS ; i++)
    m_flgCellArea[i] = false;
    
  m_deleteButton->Disable();
  
  m_mgh->DrawRectangle(0, Y_RECRES - H_RECSPC, SCREEN_WIDTH - 1, SCREEN_HEIGHT - 12, SCR_BACK, true);
}
    
/*
 * Handle recognized char
 */
void CellWriterWriteForm::HandleRecognizedChar()
{
  /* debug for memory leaks 
  
  char buff[80];
  
  void *p = malloc(10000);
  
  sprintf(buff, "0x%x", (int)p);
  
  free(p);
  
  m_mgh->DrawRectangle(150, 0, 250, 15, SCR_BACK, true);
  m_mgh->DrawTextTrueType(GraphicsHandler::TTF_goulong, buff, 150, 15, 10, TXT_COLOR, SCR_BACK);
  */
  
  /* Display recognition result */

  if (m_ch != -1)
  {
    DrawCell(m_mgh, (X_WRFIRSTCELL + m_curCellArea * W_WRCELL), Y_WRCELLRES, m_ch, CELL_DRAW);
    
    int n = m_eng->GetNumRecognitionAlts();
    
    int ch, nSample, pct;
    
    for (int i=0; i<n; i++)
    {
      m_eng->GetRecognitionAltInfo(i, &ch, &nSample, &pct);
      
      if (i == 0)
      {
        /* Show ink */
        
        m_eng->DrawSample(ch, nSample, true, (X_WRFIRSTCELL + m_curCellArea * W_WRCELL), Y_WRCELLRES, W_CELL, H_CELL, CELL_INK);
      }
      
      ShowRecognitionResult(m_curCellArea, i, (char)ch, pct);
    }
  }
  else
    DrawCell(m_mgh, (X_WRFIRSTCELL + m_curCellArea * W_WRCELL), Y_WRCELLRES, (int)'?', CELL_RECERR);
  
  /* Next char */
  
  m_flgCellArea[m_curCellArea] = true;    // used
  
  m_curCellArea = -1;
  
  /* Enable delete button */
  
  if (!m_deleteButton->IsEnabled())
    m_deleteButton->Enable();
}

/*
 * Check if requested point is into cell area
 */
bool CellWriterWriteForm::IsInCellArea(int x, int y)
{
  if (y < Y_WRCELLINP  ||  y > (Y_WRCELLINP + H_CELL))
    return (false);
    
  if (m_curCellArea != -1)
  {
    /* Check if in the same cell area */

    int xc = X_WRFIRSTCELL + m_curCellArea * W_WRCELL;
    
    return (x >= xc  &&  x <= (xc + W_CELL));
  }
  
  /* Check another cell area (not already used) */
  
  for (int i=0; i<NWRCELLS; i++)
  {
    int xc = X_WRFIRSTCELL + i * W_WRCELL;
    
    if (x >= xc  &&  x <= (xc + W_CELL))
    {
      if (m_flgCellArea[i])
        return (false);   // already used
      else
      {
        m_curCellArea = i;
        
        return (true);
      }
    }
  }
  
  return (false);
}

/*
 * Enable/disable numbers
 */
void CellWriterWriteForm::EnableNumbers(bool bEnable)
{
  m_eng->EnableSampleRange((int)'0', (int)'9', bEnable);
}

/*
 * Enable/disable capital letters
 */
void CellWriterWriteForm::EnableCapLetters(bool bEnable)
{
  m_eng->EnableSampleRange((int)'A', (int)'Z', bEnable);
}

/*
 * Enable/disable letters
 */
void CellWriterWriteForm::EnableLetters(bool bEnable)
{
  m_eng->EnableSampleRange((int)'a', (int)'z', bEnable);
}

/*
 * Enable/disable symbols
 */
void CellWriterWriteForm::EnableSymbols(bool bEnable)
{
  m_eng->EnableSampleRange(0x21, 0x2F, bEnable);
  m_eng->EnableSampleRange(0x3A, 0x40, bEnable);
  m_eng->EnableSampleRange(0x5B, 0x60, bEnable);
  m_eng->EnableSampleRange(0x7B, 0x7E, bEnable);
}

/*
 * Enable/disable custom symbols
 */
void CellWriterWriteForm::EnableCustoms(bool bEnable)
{
  m_eng->EnableSampleRange(0x7F, 0x8E, bEnable);
}
