
/* CellWriterWriteForm declaration 

Copyright (C) 2009 Roberto Navone <ron@object65.it>
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; version 2
of the License.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
*/

#ifndef CELLWRITERWRITEFORMH
#define CELLWRITERWRITEFORMH

#include "CheckBoxesHandler.h"
#include "CellWriterForm.h"

class CellWriterWriteForm : public CellWriterForm
{
public:

  /*
   * Constructor
   */
  CellWriterWriteForm(CellWriterEngine* eng, MainGraphicsHandler* mgh, SubGraphicsHandler* sgh, TimersHandler* th);
  
protected:

  /*
   * Initialize the form 
   */
  void InitForm();
  
  /*
   * Check if requested command is form's exit command
   */
  bool IsExitCommand(int cmd);
  
  /*
   * Check if requested command is form's change mode command
   */
  bool IsChangeModeCommand(int cmd);
  
  /*
   * Return the timeout command code
   */
  int GetTimeoutCommand();
  
  /*
   * Handle the requested command
   */
  void HandleCommand(int cmd);
  
  /*
   * Handle a button press event in cell area
   */
  void HandleButtonPressEvent(int x, int y);
  
  /*
   * Handle a button release event in cell area
   */
  void HandleButtonReleaseEvent();
  
  /*
   * Handle a motion notify event in cell area
   */
  void HandleMotionNotifyEvent(int x, int y);
  
  /*
   * Handle a finish timeout event in cell area
   */
  void HandleFinishTimeout();
  
  /*
   * Check if requested point is into cell area
   */
  bool IsInCellArea(int x, int y);
  
  /*
   * Get timeout for symbol sampling 
   */
  int GetSamplingTimeout() { return (30); } // 500 msec
  
private:

  /*
   * Show recognition result
   */
  void ShowRecognitionResult(int nchar, int nalt, char c, int pct);
  
  /*
   * Show recognition time
   */
  void ShowRecognitionTime(int msec);
  
  /*
   * Clear all recognition results
   */
  void ClearRecognitionResults();
  
  /*
   * Commands
   */
  enum Command
  {
    IDC_TRAIN,
    IDC_DELETE,
    IDC_TIMEOUT,
    IDC_CBXNUMBERS,
    IDC_CBXCAPLETTERS,
    IDC_CBXLETTERS,
    IDC_CBXSYMBOLS,
    IDC_CBXCUSTOM,
  };
  
  /*
   * Delete current draw
   */
  void PerformDrawDelete();
      
  /*
   * Handle recognized char
   */
  void HandleRecognizedChar();
  
  /*
   * Enable/disable numbers
   */
  void EnableNumbers(bool bEnable);
  
  /*
   * Enable/disable capital letters
   */
  void EnableCapLetters(bool bEnable);
  
  /*
   * Enable/disable letters
   */
  void EnableLetters(bool bEnable);
  
  /*
   * Enable/disable symbols
   */
  void EnableSymbols(bool bEnable);
  
  /*
   * Enable/disable custom symbols
   */
  void EnableCustoms(bool bEnable);
  
  /*
   * Delete button
   */
  Button *m_deleteButton;
  
  /*
   * Current cell area
   */
  int m_curCellArea;
  
  /*
   * Cell areas flag
   */
  bool* m_flgCellArea;
  
  /*
   * Recognized char
   */
  int m_ch;
  
};

#ifdef TRYSCRDEFS

#define X_TITLE   4
#define Y_TITLE   22
#define H_TITLE   20

#define NWRCELLS  6

#define X_WRFIRSTCELL   4
#define W_WRCELL        42
#define Y_WRCELLRES     30
#define Y_WRCELLINP     70

#define Y_RECRES    115
#define H_RECRES    10
#define H_RECSPC    14

#define X_BOTBTN1   4
#define X_BOTBTN2   88
#define X_BOTBTN3   172
#define Y_BOTBTNS   160
#define W_BOTBTNS   80
#define H_BOTBTNS   30

#define X_CELL      108
#define Y_UPCELL    40
#define Y_DNCELL    90
#define W_CELL      40
#define H_CELL      70
#define H_UPCELL    50
#define H_DNCELL    20

//#define X_RECRES    180
//#define Y_RECRES    40
#define X_RECTM     160

#define X_IDBTNS    10
#define Y_IKBTN     75
#define Y_DLBTN     45
#define W_IDBTNS    80
#define H_IDBTNS    20

#define X_ENABLEDLIT      10
#define Y_ENABLEDLIT      15
#define H_ENABLEDLIT      12

#define X_CBXNUMBERS      10
#define Y_CBXNUMBERS      25
#define X_CBXCAPLETTERS   100
#define Y_CBXCAPLETTERS   25
#define X_CBXLETTERS      190
#define Y_CBXLETTERS      25
#define X_CBXSYMBOLS      10
#define Y_CBXSYMBOLS      45
#define X_CBXCUSTOM       100
#define Y_CBXCUSTOM       45

#define X_NUMBERS      30
#define Y_NUMBERS      35
#define X_CAPLETTERS   120
#define Y_CAPLETTERS   35
#define X_LETTERS      210
#define Y_LETTERS      35
#define X_SYMBOLS      30
#define Y_SYMBOLS      55
#define X_CUSTOM       120
#define Y_CUSTOM       55

#endif
#endif
