
/* CheckBoxesHandler implementation

Copyright (C) 2009 Roberto Navone <ron@object65.it>
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; version 2
of the License.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
*/

#include <nds.h>
#include <stdio.h>

#define CHECKBOXESDATA
#include "CheckBoxesHandler.h"

/*
 * Constructor
 */
CheckBox::CheckBox(int id, int x, int y, bool bChecked, GraphicsHandler* gh) : Control(id, x, y, gh)
{
  m_bChecked = bChecked;
  
  Draw();
}

/*
 * Destructor
 */
CheckBox::~CheckBox()
{
}

/*
 * Set checked state
 */
void CheckBox::SetCheck(bool bChecked)
{
  if (bChecked == m_bChecked)
    return;
    
  m_bChecked = bChecked;
  
  Draw();
}

/*
 * Draw the checkbox
 */
void CheckBox::Draw()
{
  m_gh->DrawRectangle(m_x, m_y, (m_x + CBXW), (m_y + CBXH), CBX_BACK, true);
  
  if (m_bChecked)
  {
    m_gh->DrawRectangle(m_x+3, m_y+3, (m_x + CBXW - 3), (m_y + CBXH - 3), CBX_CLR1, true);
    m_gh->DrawPoint(m_x+3, m_y+3, CBX_CLR2);
    m_gh->DrawPoint(m_x+CBXW-3, m_y+3, CBX_CLR2);
    m_gh->DrawPoint(m_x+3, m_y+CBXH-4, CBX_CLR2);
    m_gh->DrawPoint(m_x+CBXW-3, m_y+CBXH-4, CBX_CLR2);
    m_gh->DrawLine(m_x+4, m_y+2, (m_x + CBXW - 4), m_y+2, CBX_CLR2);
    m_gh->DrawLine(m_x+4, (m_y + CBXH - 3), (m_x + CBXW - 4), (m_y + CBXH - 3), CBX_CLR2);
    m_gh->DrawLine(m_x+2, m_y+4, m_x+2, (m_y + CBXH - 5), CBX_CLR2);
    m_gh->DrawLine((m_x + CBXW - 2), m_y+4, (m_x + CBXW - 2), (m_y + CBXH - 5), CBX_CLR2);
  }
}

/*
 * Constructor
 */
CheckBoxesHandler::CheckBoxesHandler(GraphicsHandler* gh)
{
  m_gh = gh;
  
  for (int i=0; i<MAXCHECKBOXES; i++)
    m_checkboxes[i] = 0;
}

/*
 * Destructor
 */
CheckBoxesHandler::~CheckBoxesHandler()
{
  Reset();
}

/*
 * Add the requested checkbox
 */
CheckBox* CheckBoxesHandler::AddCheckBox(int id, int x, int y, bool bChecked)
{
  int n = -1;
  
  for (int i=0; i<MAXCHECKBOXES; i++)
  {
    if (m_checkboxes[i] != 0)
    {
      if (m_checkboxes[i]->GetId() == id)
        return (m_checkboxes[i]);
    }
    else if (n == -1)
      n = i;
  }
  
  if (n != -1)
    m_checkboxes[n] = new CheckBox(id, x, y, bChecked, m_gh);
    
  return (m_checkboxes[n]);
}


/*
 * Remove the requested checkbox
 */
void CheckBoxesHandler::RemoveCheckBox(int id)
{
  for (int i=0; i<MAXCHECKBOXES; i++)
    if (m_checkboxes[i] != 0)
    {
      if (m_checkboxes[i]->GetId() == id)
      {
        delete m_checkboxes[i];
        
        m_checkboxes[i] = 0;
        
        break;
      }
    }  
}

/*
 * Check for checkbox presence at given position
 */
CheckBox* CheckBoxesHandler::CheckCheckBoxAt(int x, int y)
{
  for (int i=0; i<MAXCHECKBOXES; i++)
    if (m_checkboxes[i] != 0)
    {
      CheckBox* cbx = m_checkboxes[i];
      
      if (cbx->IsEnabled())
      {
        if (x >= cbx->GetX()  &&  x <= (cbx->GetX() + CBXW)  &&
            y >= cbx->GetY()  &&  y <= (cbx->GetY() + CBXH))
          return (cbx);
      }
    }
    
  return (0);
}

/*
 * Retrieve the requested checkbox
 */
CheckBox* CheckBoxesHandler::GetCheckBox(int id)
{
  for (int i=0; i<MAXCHECKBOXES; i++)
    if (m_checkboxes[i] != 0)
      if (m_checkboxes[i]->GetId() == id)
        return (m_checkboxes[i]);
    
  return (0);
}

/*
 * Delete all checkboxes
 */
void CheckBoxesHandler::Reset()
{
  for (int i=0; i<MAXCHECKBOXES; i++)
    if (m_checkboxes[i] != 0)
    {
      delete m_checkboxes[i];
      
      m_checkboxes[i] = 0;
    }  
}


