
/* CheckBoxesHandler declaration

Copyright (C) 2009 Roberto Navone <ron@object65.it>
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; version 2
of the License.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
*/

#ifndef CHECKBOXHANDLERH
#define CHECKBOXHANDLERH

#include "GraphicsHandler.h"
#include "Control.h"

class CheckBox : public Control
{
friend class CheckBoxesHandler;

protected:

  /*
   * Constructor
   */
  CheckBox(int id, int x, int y, bool bCkecked, GraphicsHandler* gh);
  
  /*
   * Destructor
   */
  ~CheckBox();
  
public:

  /*
   * Return checked state
   */
  bool IsChecked() { return (m_bChecked); }
  
  /*
   * Set checked state
   */
  void SetCheck(bool bChecked);
  
private:

  /*
   * Draw the checkbox
   */
  void Draw();
  
  /*
   * Checked flag
   */
  bool m_bChecked;

};

#define MAXCHECKBOXES    10

class CheckBoxesHandler
{
public:

  /*
   * Constructor
   */
  CheckBoxesHandler(GraphicsHandler* gh);
  
  /*
   * Destructor
   */
  ~CheckBoxesHandler();
  
  /*
   * Add the requested checkbox
   */
  CheckBox* AddCheckBox(int id, int x, int y, bool bCkecked);

  /*
   * Remove the requested checkbox
   */
  void RemoveCheckBox(int id);
  
  /*
   * Check for checkbox presence at given position
   */
  CheckBox* CheckCheckBoxAt(int x, int y);
  
  /*
   * Retrieve the requested checkbox
   */
  CheckBox* GetCheckBox(int id);
  
  /*
   * Delete all checkboxes
   */
  void Reset();

private:

  /*
   * CheckBoxes defined
   */
  CheckBox* m_checkboxes[MAXCHECKBOXES];
  
  /*
   * Graphics handler reference
   */
  GraphicsHandler* m_gh;

};

#ifdef CHECKBOXESDATA

#define CBXW  12
#define CBXH  12

#define CBX_BACK    RGB15(31,31,31)
#define CBX_CLR1    RGB15(0, 0, 0)
#define CBX_CLR2    RGB15(10, 10, 10)

#endif

#endif

